#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Training Executor class."""

import os
from argparse import Namespace
import pytorch_lightning as pl
import torch
from pytorch_kfp_components.components.trainer.generic_executor import (
    GenericExecutor,
)
from pytorch_kfp_components.types import standard_component_specs


class Executor(GenericExecutor):
    """The Training Executor class."""

    def __init__(self):  # pylint:disable=useless-super-delegation
        super().__init__()

    def Do(self, input_dict: dict, output_dict: dict, exec_properties: dict):  #pylint: disable=too-many-locals
        """This function of the Executor invokes the PyTorch Lightning training
        loop.

        Args:
            input_dict : The dictionary of inputs.Example
            : model file, data module file
            output_dict :
            exec_properties : A dict of execution properties
                            including data_module_args,
                             trainer_args, module_file_args

        Returns:
            trainer : The object of PyTorch-Lightning Trainer.

        Raises:
            ValueError : If both of module_file_arfs or trainer_args are empty.
            TypeError : If the type of trainer_args is not dict.
            NotImplementedError : If mandatory args;
                                module_file or data_module_file is empty.
        """
        self._log_startup(
            input_dict=input_dict,
            output_dict=output_dict,
            exec_properties=exec_properties,
        )

        (
            module_file,
            data_module_file,
            trainer_args,
            module_file_args,
            data_module_args,
        ) = self._get_fn_args(
            input_dict=input_dict,
            output_dict=output_dict,
            execution_properties=exec_properties,
        )

        (
            model_class,
            data_module_class,
        ) = self.derive_model_and_data_module_class(
            module_file=module_file, data_module_file=data_module_file
        )
        if not data_module_class :
            raise NotImplementedError(
                "Data module class is mandatory. "
                "User defined training module is yet to be supported."
            )
        if data_module_class:
            data_module = data_module_class(
                **data_module_args if data_module_args else {}
            )
            data_module.prepare_data()
            data_module.setup(stage="fit")
            model = model_class(**module_file_args if module_file_args else {})

            if (not module_file_args) and (not trainer_args):
                raise ValueError("Module file & trainer args can't be empty")

            if not isinstance(trainer_args, dict):
                raise TypeError("trainer_args must be a dict")

            module_file_args.update(trainer_args)
            parser = Namespace(**module_file_args)
            trainer = pl.Trainer.from_argparse_args(parser)

            trainer.fit(model, data_module)  #pylint: disable=no-member
            trainer.test(model)  #pylint: disable=no-member

            if "checkpoint_dir" in module_file_args:
                model_save_path = module_file_args["checkpoint_dir"]
            else:
                model_save_path = "/tmp"

            if "model_name" in module_file_args:
                model_name = module_file_args["model_name"]
            else:
                model_name = "model_state_dict.pth"

            model_save_path = os.path.join(model_save_path, model_name)
            if trainer.global_rank == 0:
                print("Saving model to {}".format(model_save_path))
                torch.save(model.state_dict(), model_save_path)

            output_dict[standard_component_specs.TRAINER_MODEL_SAVE_PATH
                       ] = model_save_path
            output_dict[standard_component_specs.PTL_TRAINER_OBJ] = trainer



#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Pipeline Base Executor class."""
import abc
import logging
from six import with_metaclass


class BaseExecutor(with_metaclass(abc.ABCMeta, object)):  # pylint: disable=R0903
    """Pipeline Base Executor abstract class."""

    def __init__(self):
        pass  # pylint: disable=W0107

    @abc.abstractmethod
    def Do(self, input_dict: dict, output_dict: dict, exec_properties: dict):  # pylint: disable=C0103
        """A Do function that does nothing."""
        pass  # pylint: disable=W0107

    def _log_startup(
        self, input_dict: dict, output_dict: dict, exec_properties
    ):
        """Log inputs, outputs, and executor properties in a standard
        format."""
        class_name = self.__class__.__name__
        logging.debug("Starting %s execution.", class_name)
        logging.debug("Inputs for %s are: %s .", class_name, input_dict)
        logging.debug("Outputs for %s are: %s.", class_name, output_dict)
        logging.debug(
            "Execution Properties for %s are: %s",
            class_name, exec_properties)



#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Visualization Executor Class."""
# pylint: disable=C0103
# pylint: disable=R0201
import json
import os
import tempfile
from pathlib import Path
from urllib.parse import urlparse

import pandas as pd
from sklearn.metrics import confusion_matrix

from pytorch_kfp_components.components.base.base_executor import BaseExecutor
from pytorch_kfp_components.components.minio.component import MinIO
from pytorch_kfp_components.types import standard_component_specs


class Executor(BaseExecutor):  # pylint: disable=R0903
    """Visualization Executor Class."""

    def __init__(self):
        super(Executor, self).__init__()  # pylint: disable=R1725
        self.mlpipeline_ui_metadata = None
        self.mlpipeline_metrics = None

    def _write_ui_metadata(
        self, metadata_filepath, metadata_dict, key="outputs"
    ):
        """Function to write the metadata to UI."""
        if not os.path.exists(metadata_filepath):
            metadata = {key: [metadata_dict]}
        else:
            with open(metadata_filepath) as fp:
                metadata = json.load(fp)
                metadata_outputs = metadata[key]
                metadata_outputs.append(metadata_dict)

        print("Writing to file: {}".format(metadata_filepath))
        with open(metadata_filepath, "w") as fp:
            json.dump(metadata, fp)

    def _generate_markdown(self, markdown_dict):
        """Generates a markdown.

        Args:
            markdown_dict : dict of markdown specifications
        """
        source_str = json.dumps(
            markdown_dict["source"], sort_keys=True, indent=4
        )
        source = f"```json \n {source_str} ```"
        markdown_metadata = {
            "storage": markdown_dict["storage"],
            "source": source,
            "type": "markdown",
        }

        self._write_ui_metadata(
            metadata_filepath=self.mlpipeline_ui_metadata,
            metadata_dict=markdown_metadata,
        )

    def _generate_confusion_matrix_metadata(
        self, confusion_matrix_path, classes
    ):
        """Generates the confusion matrix metadata and writes in ui."""
        print("Generating Confusion matrix Metadata")
        metadata = {
            "type": "confusion_matrix",
            "format": "csv",
            "schema": [
                {"name": "target", "type": "CATEGORY"},
                {"name": "predicted", "type": "CATEGORY"},
                {"name": "count", "type": "NUMBER"},
            ],
            "source": confusion_matrix_path,
            "labels": list(map(str, classes)),
        }

        self._write_ui_metadata(
            metadata_filepath=self.mlpipeline_ui_metadata,
            metadata_dict=metadata,
        )

    def _upload_confusion_matrix_to_minio(
        self, confusion_matrix_url, confusion_matrix_output_path
    ):
        """Uploads the generated confusion matrix to minio"""
        parse_obj = urlparse(confusion_matrix_url, allow_fragments=False)
        bucket_name = parse_obj.netloc
        folder_name = str(parse_obj.path).lstrip("/")

        # TODO:  # pylint: disable=W0511
        endpoint = "minio-service.kubeflow:9000"
        MinIO(
            source=confusion_matrix_output_path,
            bucket_name=bucket_name,
            destination=folder_name,
            endpoint=endpoint,
        )

    def _generate_confusion_matrix(
        self, confusion_matrix_dict
    ):  # pylint: disable=R0914
        """Generates confusion matrix in minio."""
        actuals = confusion_matrix_dict["actuals"]
        preds = confusion_matrix_dict["preds"]
        confusion_matrix_url = confusion_matrix_dict["url"]

        # Generating confusion matrix
        df = pd.DataFrame(
            list(zip(actuals, preds)), columns=["target", "predicted"]
        )
        vocab = list(df["target"].unique())
        cm = confusion_matrix(df["target"], df["predicted"], labels=vocab)
        data = []
        for target_index, target_row in enumerate(cm):
            for predicted_index, count in enumerate(target_row):
                data.append(
                    (vocab[target_index], vocab[predicted_index], count)
                )

        confusion_matrix_df = pd.DataFrame(
            data, columns=["target", "predicted", "count"]
        )

        confusion_matrix_output_dir = str(tempfile.mkdtemp())
        confusion_matrix_output_path = os.path.join(
            confusion_matrix_output_dir, "confusion_matrix.csv"
        )
        # saving confusion matrix
        confusion_matrix_df.to_csv(
            confusion_matrix_output_path, index=False, header=False
        )

        self._upload_confusion_matrix_to_minio(
            confusion_matrix_url=confusion_matrix_url,
            confusion_matrix_output_path=confusion_matrix_output_path,
        )

        # Generating metadata
        self._generate_confusion_matrix_metadata(
            confusion_matrix_path=os.path.join(
                confusion_matrix_url, "confusion_matrix.csv"
            ),
            classes=vocab,
        )

    def _visualize_accuracy_metric(self, accuracy):
        """Generates the visualization for accuracy."""
        metadata = {
            "name": "accuracy-score",
            "numberValue": accuracy,
            "format": "PERCENTAGE",
        }
        self._write_ui_metadata(
            metadata_filepath=self.mlpipeline_metrics,
            metadata_dict=metadata,
            key="metrics",
        )

    def _get_fn_args(self, input_dict: dict, exec_properties: dict):
        """Extracts the confusion matrix dict, test accuracy, markdown from the
        input dict and mlpipeline ui metadata & metrics from exec_properties.

        Args:
            input_dict : a dictionary of inputs,
                        example: confusion matrix dict, markdown
            exe_properties : a dict of execution properties
                           example : mlpipeline_ui_metadata
        Returns:
            confusion_matrix_dict : dict of confusion metrics
            test_accuracy : model test accuracy metrics
            markdown : markdown dict
            mlpipeline_ui_metadata : path of ui metadata
            mlpipeline_metrics : metrics to be uploaded
        """
        confusion_matrix_dict = input_dict.get(
            standard_component_specs.VIZ_CONFUSION_MATRIX_DICT
        )
        test_accuracy = input_dict.get(
            standard_component_specs.VIZ_TEST_ACCURACY
        )
        markdown = input_dict.get(standard_component_specs.VIZ_MARKDOWN)

        mlpipeline_ui_metadata = exec_properties.get(
            standard_component_specs.VIZ_MLPIPELINE_UI_METADATA
        )
        mlpipeline_metrics = exec_properties.get(
            standard_component_specs.VIZ_MLPIPELINE_METRICS
        )

        return (
            confusion_matrix_dict,
            test_accuracy,
            markdown,
            mlpipeline_ui_metadata,
            mlpipeline_metrics,
        )

    def _set_defalt_mlpipeline_path(
        self, mlpipeline_ui_metadata: str, mlpipeline_metrics: str
    ):
        """Sets the default mlpipeline path."""

        if mlpipeline_ui_metadata:
            Path(os.path.dirname(mlpipeline_ui_metadata)).mkdir(
                parents=True, exist_ok=True
            )
        else:
            mlpipeline_ui_metadata = "/mlpipeline-ui-metadata.json"

        if mlpipeline_metrics:
            Path(os.path.dirname(mlpipeline_metrics)).mkdir(
                parents=True, exist_ok=True
            )
        else:
            mlpipeline_metrics = "/mlpipeline-metrics.json"

        return mlpipeline_ui_metadata, mlpipeline_metrics

    def Do(self, input_dict: dict, output_dict: dict, exec_properties: dict):
        """Executes the visualization process and uploads to minio
        Args:
             input_dict : a dictionary of inputs,
                         example: confusion matrix dict, markdown
             output_dict :
             exec_properties : a dict of execution properties
                            example : mlpipeline_ui_metadata
        """

        (
            confusion_matrix_dict,
            test_accuracy,
            markdown,
            mlpipeline_ui_metadata,
            mlpipeline_metrics,
        ) = self._get_fn_args(
            input_dict=input_dict, exec_properties=exec_properties
        )

        (
            self.mlpipeline_ui_metadata,
            self.mlpipeline_metrics,
        ) = self._set_defalt_mlpipeline_path(
            mlpipeline_ui_metadata=mlpipeline_ui_metadata,
            mlpipeline_metrics=mlpipeline_metrics,
        )

        if not (confusion_matrix_dict or test_accuracy or markdown):
            raise ValueError(
                "Any one of these keys should be set - "
                "confusion_matrix_dict, test_accuracy, markdown"
            )

        if confusion_matrix_dict:
            self._generate_confusion_matrix(
                confusion_matrix_dict=confusion_matrix_dict,
            )

        if test_accuracy:
            self._visualize_accuracy_metric(accuracy=test_accuracy)

        if markdown:
            self._generate_markdown(markdown_dict=markdown)

        output_dict[
            standard_component_specs.VIZ_MLPIPELINE_UI_METADATA
        ] = self.mlpipeline_ui_metadata
        output_dict[
            standard_component_specs.VIZ_MLPIPELINE_METRICS
        ] = self.mlpipeline_metrics



#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Mar Generation Component Class."""

from pytorch_kfp_components.components.mar.executor import Executor
from pytorch_kfp_components.types import standard_component_specs
from pytorch_kfp_components.components.base.base_component import BaseComponent


class MarGeneration(BaseComponent):  #pylint: disable=R0903
    """Mar generation class."""

    def __init__(self, mar_config: dict, mar_save_path: str = None):
        """Initializes the Mar Generation class.

        Args:
            mar_config: mar configuration dict (type:dict)
            mar_save_path : the path for saving the mar file (type:str)
        """
        super(MarGeneration, self).__init__()  #pylint: disable=R1725
        input_dict = {
            standard_component_specs.MAR_GENERATION_CONFIG: mar_config,
        }

        output_dict = {}

        exec_properties = {
            standard_component_specs.MAR_GENERATION_SAVE_PATH: mar_save_path
        }

        spec = standard_component_specs.MarGenerationSpec()
        self._validate_spec(
            spec=spec,
            input_dict=input_dict,
            output_dict=output_dict,
            exec_properties=exec_properties,
        )

        self._validate_mar_config_spec(spec=spec, mar_config=mar_config)

        Executor().Do(
            input_dict=input_dict,
            output_dict=output_dict,
            exec_properties=exec_properties,
        )
        self.output_dict = output_dict

    def _validate_mar_config_spec(
        self, spec: standard_component_specs, mar_config: dict
    ):
        """Validates mar config dictionary specs type"""
        for key in mar_config:
            self._type_check(
                actual_value=mar_config[key],
                key=key.lower(),
                spec_dict=spec.MAR_CONFIG_DICT,
            )



#!/usr/bin/env/python3
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Minio Component Module."""
from pytorch_kfp_components.components.base.base_component import BaseComponent
from pytorch_kfp_components.components.minio.executor import Executor
from pytorch_kfp_components.types import standard_component_specs


class MinIO(BaseComponent):  #pylint: disable=too-few-public-methods
    """Minio Component Class."""
    def __init__(
        self,
        source: str,
        bucket_name: str,
        destination: str,
        endpoint: str,
    ):
        """Initializes the component class.

        Args:
            source : the source path of artifacts.
            bucket_name : minio bucket name.
            destination : the destination path in minio
            endpoint : minio endpoint url.
        """
        super(BaseComponent, self).__init__()  #pylint: disable=bad-super-call

        input_dict = {
            standard_component_specs.MINIO_SOURCE: source,
            standard_component_specs.MINIO_BUCKET_NAME: bucket_name,
            standard_component_specs.MINIO_DESTINATION: destination,
        }

        output_dict = {}

        exec_properties = {
            standard_component_specs.MINIO_ENDPOINT: endpoint,
        }

        spec = standard_component_specs.MinIoSpec()
        self._validate_spec(
            spec=spec,
            input_dict=input_dict,
            output_dict=output_dict,
            exec_properties=exec_properties,
        )

        Executor().Do(
            input_dict=input_dict,
            output_dict=output_dict,
            exec_properties=exec_properties,
        )

        self.output_dict = output_dict



#!/usr/bin/env/python3
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Minio Executor Module."""
import os
import urllib3
from minio import Minio  #pylint: disable=no-name-in-module
from pytorch_kfp_components.components.base.base_executor import BaseExecutor
from pytorch_kfp_components.types import standard_component_specs


class Executor(BaseExecutor):
    """Minio Executor Class."""
    def __init__(self):  #pylint: disable=useless-super-delegation
        super(Executor, self).__init__()  #pylint: disable=super-with-arguments

    def _initiate_minio_client(self, minio_config: dict):  #pylint: disable=no-self-use
        """Initializes the minio client.

        Args:
            minio_config : a dict for minio configuration.
        Returns:
            client : the minio server client
        """
        minio_host = minio_config["HOST"]
        access_key = minio_config["ACCESS_KEY"]
        secret_key = minio_config["SECRET_KEY"]
        client = Minio(
            minio_host,
            access_key=access_key,
            secret_key=secret_key,
            secure=False,
        )
        return client

    def _read_minio_creds(self, endpoint: str):  #pylint: disable=no-self-use
        """Reads the minio credentials.

        Args:
            endpoint : minio endpoint url
        Raises:
            ValueError : if minio access key and secret keys are missing
        Returns:
            minio_config : a dict for minio configuration.
        """
        if "MINIO_ACCESS_KEY" not in os.environ:
            raise ValueError("Environment variable MINIO_ACCESS_KEY not found")

        if "MINIO_SECRET_KEY" not in os.environ:
            raise ValueError("Environment variable MINIO_SECRET_KEY not found")

        minio_config = {
            "HOST": endpoint,
            "ACCESS_KEY": os.environ["MINIO_ACCESS_KEY"],
            "SECRET_KEY": os.environ["MINIO_SECRET_KEY"],
        }

        return minio_config

    def upload_artifacts_to_minio(  #pylint: disable=no-self-use,too-many-arguments
        self,
        client: Minio,
        source: str,
        destination: str,
        bucket_name: str,
        output_dict: dict,
    ):
        """Uploads artifacts to minio server.

        Args:
            client : Minio client
            source : source path of artifacts.
            destination : destination path of artifacts
            bucket_name : minio bucket name.
            output_dict : dict of output containing destination paths,
                          source and bucket names
        Raises:
            Exception : on MaxRetryError, NewConnectionError,
                        ConnectionError.
        Returns:
            output_dict : dict of output containing destination paths,
                          source and bucket names
        """
        print(f"source {source} destination {destination}")
        try:
            client.fput_object(
                bucket_name=bucket_name,
                file_path=source,
                object_name=destination,
            )
            output_dict[destination] = {
                "bucket_name": bucket_name,
                "source": source,
            }
        except (
                urllib3.exceptions.MaxRetryError,
                urllib3.exceptions.NewConnectionError,
                urllib3.exceptions.ConnectionError,
                RuntimeError,
        ) as expection_raised:
            print(str(expection_raised))
            raise Exception(expection_raised)  #pylint: disable=raise-missing-from

        return output_dict

    def get_fn_args(self, input_dict: dict, exec_properties: dict):  #pylint: disable=no-self-use
        """Extracts the source, bucket_name, folder_name from the input_dict
        and endpoint from exec_properties.

        Args:
            input_dict : a dict of inputs having source,
                        destination etc.
            exec_properties : a dict of execution properties,
                                having minio endpoint.
        Returns:
            source : source path of artifacts.
            bucket_name : name of minio bucket
            folder_name : name of folder in which artifacts are uploaded.
            endpoint : minio endpoint url.
        """
        source = input_dict.get(standard_component_specs.MINIO_SOURCE)
        bucket_name = input_dict.get(
            standard_component_specs.MINIO_BUCKET_NAME)
        folder_name = input_dict.get(
            standard_component_specs.MINIO_DESTINATION)
        endpoint = exec_properties.get(standard_component_specs.MINIO_ENDPOINT)
        return source, bucket_name, folder_name, endpoint

    def Do(self, input_dict: dict, output_dict: dict, exec_properties: dict):  #pylint: disable=too-many-locals
        """Executes the minio upload process.

        Args:
            input_dict : a dict of inputs having source, destination etc.
            output_dict : dict of output containing destination paths,
                          source and bucket names
            exec_properties : a dict of execution properties,
                                having minio endpoint.

        Raises:
            ValueError : for invalid/unknonwn source path
        """

        source, bucket_name, folder_name, endpoint = self.get_fn_args(
            input_dict=input_dict, exec_properties=exec_properties)

        minio_config = self._read_minio_creds(endpoint=endpoint)

        client = self._initiate_minio_client(minio_config=minio_config)

        if not os.path.exists(source):
            raise ValueError("Input path - {} does not exists".format(source))

        if os.path.isfile(source):
            artifact_name = source.split("/")[-1]
            destination = os.path.join(folder_name, artifact_name)
            self.upload_artifacts_to_minio(
                client=client,
                source=source,
                destination=destination,
                bucket_name=bucket_name,
                output_dict=output_dict,
            )
        elif os.path.isdir(source):
            for root, dirs, files in os.walk(source):  #pylint: disable=unused-variable
                for file in files:
                    source = os.path.join(root, file)
                    artifact_name = source.split("/")[-1]
                    destination = os.path.join(folder_name, artifact_name)
                    self.upload_artifacts_to_minio(
                        client=client,
                        source=source,
                        destination=destination,
                        bucket_name=bucket_name,
                        output_dict=output_dict,
                    )
        else:
            raise ValueError("Unknown source: {} ".format(source))



#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Pipeline Base component class."""

import abc
from pytorch_kfp_components.types import standard_component_specs


class BaseComponent(metaclass=abc.ABCMeta):  # pylint: disable=R0903
    """Pipeline Base component class."""

    def __init__(self):
        pass

    @classmethod
    def _validate_spec(
        cls,
        spec: standard_component_specs,
        input_dict: dict,
        output_dict: dict,
        exec_properties: dict,
    ):
        """validate the specifications 'type'.

        Args:
            spec: The standard component specifications
            input_dict : A dictionary of inputs.
            ouput-dict :
            exec_properties : A dict of execution properties.
        """

        for key, value in input_dict.items():
            cls._type_check(
                actual_value=value, key=key, spec_dict=spec.INPUT_DICT
            )

        for key, value in output_dict.items():
            cls._type_check(
                actual_value=value, key=key, spec_dict=spec.OUTPUT_DICT
            )

        for key, value in exec_properties.items():
            cls._type_check(
                actual_value=value,
                key=key,
                spec_dict=spec.EXECUTION_PROPERTIES
            )

    @classmethod
    def _optional_check(cls, actual_value: any, key: str, spec_dict: dict):
        """Checks for optional specification.

        Args:
            actual_value : Value of the dictionary.
            key: key for the correspondin value.
            spec_dict : The dict of specification for validation.
        Returns :
            is_optional : The optional key.
        Raises :
            ValueError : If the key is not optional
        """
        is_optional = spec_dict[key].optional

        if not is_optional and not actual_value:
            raise ValueError(
                "{key} is not optional. Received value: {actual_value}".format(
                    key=key, actual_value=actual_value
                )
            )

        return is_optional

    @classmethod
    def _type_check(cls, actual_value, key, spec_dict):
        """Checks the type of specifactions.

        Args:
            actual_value : Value of the dictionary.
            key: key for the correspondin value.
            spec_dict : The dict of specification for validation.

        Raises :
            TypeError : If key value type does not match expected value type.
        """
        if not actual_value:
            is_optional = cls._optional_check(
                actual_value=actual_value, key=key, spec_dict=spec_dict
            )
            if is_optional:
                return

        expected_type = spec_dict[key].type
        actual_type = type(actual_value)
        if actual_type != expected_type:
            raise TypeError(
                "{key} must be of type {expected_type} "
                "but received as {actual_type}"
                .format(
                    key=key,
                    expected_type=expected_type,
                    actual_type=actual_type,
                )
            )



#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Module for defining standard specifications and validation of parameter
type."""
#pylint: disable=duplicate-code
TRAINER_MODULE_FILE = "module_file"
TRAINER_DATA_MODULE_FILE = "data_module_file"
TRAINER_DATA_MODULE_ARGS = "data_module_args"
TRAINER_MODULE_ARGS = "module_file_args"
PTL_TRAINER_ARGS = "trainer_args"

TRAINER_MODEL_SAVE_PATH = "model_save_path"
PTL_TRAINER_OBJ = "ptl_trainer"

MAR_GENERATION_CONFIG = "mar_config"
MAR_GENERATION_SAVE_PATH = "mar_save_path"
CONFIG_PROPERTIES_SAVE_PATH = "config_prop_save_path"
MAR_CONFIG_MODEL_NAME = "model_name"
MAR_CONFIG_MODEL_FILE = "model_file"
MAR_CONFIG_MODEL_HANDLER = "handler"
MAR_CONFIG_SERIALIZED_FILE = "serialized_file"
MAR_CONFIG_VERSION = "version"
MAR_CONFIG_EXPORT_PATH = "export_path"
MAR_CONFIG_CONFIG_PROPERTIES = "config_properties"
MAR_CONFIG_REQUIREMENTS_FILE = "requirements_file"
MAR_CONFIG_EXTRA_FILES = "extra_files"

VIZ_MLPIPELINE_UI_METADATA = "mlpipeline_ui_metadata"
VIZ_MLPIPELINE_METRICS = "mlpipeline_metrics"
VIZ_CONFUSION_MATRIX_DICT = "confusion_matrix_dict"
VIZ_TEST_ACCURACY = "test_accuracy"

VIZ_MARKDOWN = "markdown"
VIZ_MARKDOWN_DICT_SOURCE = "source"
VIZ_MARKDOWN_DICT_STORAGE = "storage"

VIZ_CONFUSION_MATRIX_ACTUALS = "actuals"
VIZ_CONFUSION_MATRIX_PREDS = "preds"
VIZ_CONFUSION_MATRIX_CLASSES = "classes"
VIZ_CONFUSION_MATRIX_URL = "url"

MINIO_SOURCE = "source"
MINIO_BUCKET_NAME = "bucket_name"
MINIO_DESTINATION = "destination"
MINIO_ENDPOINT = "endpoint"

class Parameters:  # pylint: disable=R0903
    """Parameter class to match the desired type."""

    def __init__(self, type=None, optional=False):  # pylint: disable=redefined-builtin
        self.type = type
        self.optional = optional


class TrainerSpec:  # pylint: disable=R0903
    """Trainer Specification class.
    For validating the parameter 'type' .
    """

    INPUT_DICT = {
        TRAINER_MODULE_FILE: Parameters(type=str),
        TRAINER_DATA_MODULE_FILE: Parameters(type=str),
    }

    OUTPUT_DICT = {}

    EXECUTION_PROPERTIES = {
        TRAINER_DATA_MODULE_ARGS: Parameters(type=dict, optional=True),
        TRAINER_MODULE_ARGS: Parameters(type=dict),
        PTL_TRAINER_ARGS: Parameters(type=dict, optional=True),
    }


class MarGenerationSpec:  # pylint: disable=R0903
    """Mar Specification class.
    For validating the parameter 'type' .
    """

    INPUT_DICT = {
        MAR_GENERATION_CONFIG: Parameters(type=dict),
    }

    OUTPUT_DICT = {}

    EXECUTION_PROPERTIES = {
        MAR_GENERATION_SAVE_PATH: Parameters(type=str, optional=True),
    }

    MAR_CONFIG_DICT = {
        MAR_CONFIG_MODEL_NAME: Parameters(type=str),
        MAR_CONFIG_MODEL_FILE: Parameters(type=str),
        MAR_CONFIG_MODEL_HANDLER: Parameters(type=str),
        MAR_CONFIG_SERIALIZED_FILE: Parameters(type=str),
        MAR_CONFIG_VERSION: Parameters(type=str),
        MAR_CONFIG_EXPORT_PATH: Parameters(type=str, optional=True),
        MAR_CONFIG_CONFIG_PROPERTIES: Parameters(type=str, optional=True),
        MAR_CONFIG_REQUIREMENTS_FILE: Parameters(type=str, optional=True),
        MAR_CONFIG_EXTRA_FILES: Parameters(type=str, optional=True),
    }


class VisualizationSpec:  #pylint: disable=too-few-public-methods
    """Visualization Specification class.
    For validating the parameter 'type'
    """
    INPUT_DICT = {
        VIZ_CONFUSION_MATRIX_DICT: Parameters(type=dict, optional=True),
        VIZ_TEST_ACCURACY: Parameters(type=float, optional=True),
        VIZ_MARKDOWN: Parameters(type=dict, optional=True),
    }

    OUTPUT_DICT = {}

    EXECUTION_PROPERTIES = {
        VIZ_MLPIPELINE_UI_METADATA: Parameters(type=str, optional=True),
        VIZ_MLPIPELINE_METRICS: Parameters(type=str, optional=True),
    }

    MARKDOWN_DICT = {
        VIZ_MARKDOWN_DICT_STORAGE: Parameters(type=str, optional=False),
        VIZ_MARKDOWN_DICT_SOURCE: Parameters(type=dict, optional=False),
    }

    CONFUSION_MATRIX_DICT = {
        VIZ_CONFUSION_MATRIX_ACTUALS: Parameters(type=list, optional=False),
        VIZ_CONFUSION_MATRIX_PREDS: Parameters(type=list, optional=False),
        VIZ_CONFUSION_MATRIX_CLASSES: Parameters(type=list, optional=False),
        VIZ_CONFUSION_MATRIX_URL: Parameters(type=str, optional=False),
    }


class MinIoSpec:  #pylint: disable=too-few-public-methods
    """MinIO Specification class.
    For validating the parameter 'type'
    """
    INPUT_DICT = {
        MINIO_SOURCE: Parameters(type=str),
        MINIO_BUCKET_NAME: Parameters(type=str),
        MINIO_DESTINATION: Parameters(type=str),
    }

    OUTPUT_DICT = {}

    EXECUTION_PROPERTIES = {
        MINIO_ENDPOINT: Parameters(type=str),
    }



#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Mar Generation Executor Class."""

import os
import shutil
import tempfile
import subprocess
from pathlib import Path
import wget
from pytorch_kfp_components.components.base.base_executor import BaseExecutor
from pytorch_kfp_components.types import standard_component_specs


class Executor(BaseExecutor):
    """Mar Generation Executor Class."""

    def __init__(self):  #pylint: disable=W0235
        """Initializing the executor."""
        super(Executor, self).__init__()  #pylint: disable=R1725

    def get_fn_args(self, input_dict: dict, exec_properties: dict):  #pylint: disable=R0201
        """Extracts the mar config from the input dict and save path from
        exec_properties.

        Args:
            input_dict : A input dict, Eample : mar config dict
            exec_properties : A dict of execution properties for mar saving path
        Returns:
            mar_config : the dict of configuration for mar generation
            mar_save_path : the path for mar file saving
        """

        mar_config = input_dict.get(
            standard_component_specs.MAR_GENERATION_CONFIG
        )
        mar_save_path = exec_properties.get(
            standard_component_specs.MAR_GENERATION_SAVE_PATH
        )
        return mar_config, mar_save_path

    def _validate_mar_config(self, mar_config):  #pylint: disable=R0201
        """Validates the mar configuration properties.

        Args:
            mar_config : a dict of mar configurations.
        Raises:
            ValueError: if the config file or any mandatory arg is empty
        """

        mandatory_args = [
            "MODEL_NAME",
            "SERIALIZED_FILE",
            "MODEL_FILE",
            "HANDLER",
            "VERSION",
            "CONFIG_PROPERTIES",
        ]

        if not mar_config:
            raise ValueError(
                f"Mar config cannot be empty. "
                f"Mandatory arguments are {mandatory_args}"
            )

        missing_list = []
        for key in mandatory_args:
            if key not in mar_config:
                missing_list.append(key)

        if missing_list:
            raise ValueError(
                "Following Mandatory keys are missing in the config file {}".
                format(missing_list)
            )

    def download_config_properties(self, url):  #pylint: disable=R0201
        """Downloads the config.properties.

        Args:
            url : the path to download the config.properties
        """
        if not os.path.exists(url):
            try:
                url = wget.download(url, tempfile.mkdtemp())
            except ValueError as exc:
                raise ValueError(
                    "Unable to download config "
                    "properties file using url - {}".format(url)
                ) from exc

        return url

    def _generate_mar_file(
        self, mar_config: dict, mar_save_path: str, output_dict: dict
    ):
        """Generates the model mar file.

        Args:
            mar_config : mar configuration dict
            mar_save_path : path to save the mar file.
            output_dict : the output dict for saving the mar file.
        Raises:
            Exception :  If archiver command is unable to create mar in case of
                        return code 0.
        """

        self._validate_mar_config(mar_config=mar_config)

        for key, uri in mar_config.items():
            # uri = self._download_dependent_file(key, uri)
            mar_config[key] = uri

        archiver_cmd = (
            "torch-model-archiver --force "
            "--model-name {MODEL_NAME} "
            "--serialized-file {SERIALIZED_FILE} "
            "--model-file {MODEL_FILE} "
            "--handler {HANDLER} "
            "-v {VERSION}".format(
                MODEL_NAME=mar_config["MODEL_NAME"],
                SERIALIZED_FILE=mar_config["SERIALIZED_FILE"],
                MODEL_FILE=mar_config["MODEL_FILE"],
                HANDLER=mar_config["HANDLER"],
                VERSION=mar_config["VERSION"],
            )
        )

        if "EXPORT_PATH" in mar_config:
            export_path = mar_config["EXPORT_PATH"]
            output_dict[standard_component_specs.MAR_GENERATION_SAVE_PATH
                       ] = export_path
            if not os.path.exists(export_path):
                Path(export_path).mkdir(parents=True, exist_ok=True)

            archiver_cmd += " --export-path {EXPORT_PATH}".format(
                EXPORT_PATH=export_path
            )

        if "EXTRA_FILES" in mar_config:
            archiver_cmd += " --extra-files {EXTRA_FILES}".format(
                EXTRA_FILES=mar_config["EXTRA_FILES"]
            )

        if "REQUIREMENTS_FILE" in mar_config:
            archiver_cmd += " -r {REQUIREMENTS_FILE}".format(
                REQUIREMENTS_FILE=mar_config["REQUIREMENTS_FILE"]
            )

        print("Running Archiver cmd: ", archiver_cmd)

        with subprocess.Popen(archiver_cmd, shell=True, stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE) as proc:
            _, err = proc.communicate()
            if err:
                raise ValueError(err)

        # If user has provided the export path
        # By default, torch-model-archiver
        # generates the mar file inside the export path

        # If the user has not provieded the export path
        # mar file will be generated in the current working directory
        # The mar file needs to be moved into mar_save_path

        if "EXPORT_PATH" not in mar_config:
            mar_file_local_path = os.path.join(
                os.getcwd(), "{}.mar".format(mar_config["MODEL_NAME"])
            )
            if not Path(mar_save_path).exists():
                Path(mar_save_path).mkdir(parents=True, exist_ok=True)
            shutil.move(mar_file_local_path, mar_save_path)
            output_dict[standard_component_specs.MAR_GENERATION_SAVE_PATH
                       ] = mar_save_path

        elif mar_config["EXPORT_PATH"] != mar_save_path:
            raise Exception(
                "The export path [{}] needs to be same as mar save path [{}] ".
                format(mar_config["EXPORT_PATH"], mar_save_path)
            )

        print("Saving model file ")
        ## TODO: While separating the mar generation component from trainer # pylint: disable=W0511
        ## Create a separate url for model file
        print(
            f"copying {mar_config['MODEL_FILE']} to {mar_config['EXPORT_PATH']}"
        )
        shutil.copy(mar_config["MODEL_FILE"], mar_config["EXPORT_PATH"])

    def _save_config_properties(
        self, mar_config: dict, mar_save_path: str, output_dict: dict
    ):
        """Saves the config.properties file where the mar file is generated.

        Args :
            mar_config : dict of mar configuration
            mar_save_path : the location to save the config.properties
            output_dict : dict to assign the mar save path.
        """
        print("Downloading config properties")
        if "CONFIG_PROPERTIES" in mar_config:
            config_properties_local_path = self.download_config_properties(
                mar_config["CONFIG_PROPERTIES"]
            )
        else:
            config_properties_local_path = mar_config["CONFIG_PROPERTIES"]

        config_prop_path = os.path.join(mar_save_path, "config.properties")
        if os.path.exists(config_prop_path):
            os.remove(config_prop_path)
        shutil.move(config_properties_local_path, mar_save_path)
        output_dict[standard_component_specs.CONFIG_PROPERTIES_SAVE_PATH
                   ] = mar_save_path

    def Do(self, input_dict: dict, output_dict: dict, exec_properties: dict):
        """Executes the mar generation process.

        Args:
            input_dict : A input dict, including mar config
            output_dict :
            exec_properties : A dict of execution properties for mar generation
        """
        self._log_startup(
            input_dict=input_dict,
            output_dict=output_dict,
            exec_properties=exec_properties,
        )
        mar_config, mar_save_path = self.get_fn_args(
            input_dict=input_dict, exec_properties=exec_properties
        )
        self._validate_mar_config(mar_config=mar_config)

        self._generate_mar_file(
            mar_config=mar_config,
            mar_save_path=mar_save_path,
            output_dict=output_dict,
        )
        self._save_config_properties(
            mar_config=mar_config,
            mar_save_path=mar_save_path,
            output_dict=output_dict,
        )



#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Generic Executor Class."""
import importlib
import inspect
from pytorch_kfp_components.components.base.base_executor import BaseExecutor
from pytorch_kfp_components.types import standard_component_specs


class GenericExecutor(BaseExecutor):
    """Generic Executor Class that does nothing."""

    def Do(self, input_dict: dict, output_dict: dict, exec_properties: dict):
        #TODO: Code to train pretrained model  #pylint: disable=fixme
        pass

    def _get_fn_args(  #pylint: disable=no-self-use
        self, input_dict: dict, output_dict: dict, execution_properties: dict  #pylint: disable=unused-argument
    ):
        """Gets the input/output/execution properties from the dictionary.

        Args:
            input_dict : The dictionary of inputs.Example :
                        model file, data module file
            output_dict :
            exec_properties : A dict of execution properties including
                            data_module_args,trainer_args, module_file_args
        Returns:
            module_file : The model file name
            data_module_file : A data module file name
            trainer_args: A dictionary of trainer args
            module_file_args : A dictionary of model specific args
            data_module_args : A dictionary of data module args.
        """
        module_file = input_dict.get(
            standard_component_specs.TRAINER_MODULE_FILE
        )
        data_module_file = input_dict.get(
            standard_component_specs.TRAINER_DATA_MODULE_FILE
        )
        trainer_args = execution_properties.get(
            standard_component_specs.PTL_TRAINER_ARGS
        )
        module_file_args = execution_properties.get(
            standard_component_specs.TRAINER_MODULE_ARGS
        )
        data_module_args = execution_properties.get(
            (standard_component_specs.TRAINER_DATA_MODULE_ARGS)
        )
        return (
            module_file,
            data_module_file,
            trainer_args,
            module_file_args,
            data_module_args,
        )

    # pylint: disable=invalid-name
    def derive_model_and_data_module_class(  #pylint: disable=no-self-use
        self, module_file: str, data_module_file: str
    ):
        """Derives the model file and data modul file.

        Args :
            module_file : A model file name (type:str)
            data_module_file : A data module file name (type:str)

        Returns :
            model_class : The model class
            data_module_class : The data module class.

        Raises :
            ValueError: If the model file or data module file is empty.
        """
        model_class = None
        data_module_class = None

        class_module = importlib.import_module(module_file.split(".")[0])
        data_module = importlib.import_module(data_module_file.split(".")[0])

        for cls in inspect.getmembers(
                class_module,
                lambda member: inspect.isclass(member) and member.__module__ ==
                class_module.__name__,
        ):
            model_class = cls[1]

        if not model_class:
            raise ValueError(f"Unable to load module_file - {module_file}")

        for cls in inspect.getmembers(
                data_module,
                lambda member: inspect.isclass(member) and member.__module__ ==
                data_module.__name__,
        ):
            data_module_class = cls[1]

        if not data_module_class:
            raise ValueError(
                f"Unable to load data_module_file - {data_module_file}"
            )

        return model_class, data_module_class



# !/usr/bin/env/python3
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# pylint: disable=no-self-use,too-many-arguments,unused-argument,not-callable
""" Utilities for parsing arguments - Samples"""


def parse_input_args(input_str: str):
    """
    Utility to parse input string arguments. Returns a dictionary
    """
    output_dict = {}
    if not input_str:
        raise ValueError("Empty input string: {}".format(input_str))

    key_pairs: list = input_str.split(",")

    key_pairs = [x.strip() for x in key_pairs]

    if not key_pairs:
        raise ValueError("Incorrect format: {}".format(input_str))

    for each_key in key_pairs:
        try:
            key, value = each_key.split("=")
        except ValueError as value_error:
            raise ValueError("Expected input format "
                             "'key1=value1, key2=value2' "
                             "but received {}".format(input_str)) \
                from value_error
        if value.isdigit():
            value = int(value)
        output_dict[key] = value

    return output_dict



#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Setup script."""

import importlib
import os
import types

from setuptools import setup, find_packages


def make_required_install_packages():
    return [
      "kfp>=1.6.1",
      "torch>=1.7.1",
      "torchserve>=0.3.0",
      "torch-model-archiver",
    ]


def make_required_test_packages():
    return make_required_install_packages() + [
      "mock>=4.0.0",
      "flake8>=3.0.0",
      "pylint",
      "pytest>=6.0.0",
      "wget",
      "pandas",
      "minio"
    ]


def make_dependency_links():
    return ["https://github.com/PyTorchLightning/pytorch-lightning.git"]


def detect_version(base_path):
    loader = importlib.machinery.SourceFileLoader(
        fullname="version",
        path=os.path.join(base_path,
                          "pytorch_kfp_components/__init__.py"),
    )
    version = types.ModuleType(loader.name)
    loader.exec_module(version)
    return version.__version__


if __name__ == "__main__":
    relative_directory = os.path.relpath(
        os.path.dirname(os.path.abspath(__file__)))

    version = detect_version(relative_directory)

    setup(
        name="pytorch-kfp-components",
        version=version,
        description="PyTorch Kubeflow Pipeline",
        url="https://github.com/kubeflow/pipelines/tree/master/components/PyTorch/pytorch-kfp-components/",
        author="The PyTorch Kubeflow Pipeline Components authors",
        author_email="pytorch-kfp-components@fb.com",
        license="Apache License 2.0",
        extra_requires={"tests": make_required_test_packages()},
        include_package_data=True,
        python_requires=">=3.6",
        install_requires=make_required_install_packages(),
        dependency_links=make_dependency_links(),
        keywords=[
            "Kubeflow Pipelines",
            "KFP",
            "ML workflow",
            "PyTorch",
        ],
        classifiers=[
            "Development Status :: 3 - Alpha",
            "Operating System :: Unix",
            "Operating System :: MacOS",
            "Intended Audience :: Developers",
            "Intended Audience :: Education",
            "Intended Audience :: Science/Research",
            "License :: OSI Approved :: Apache Software License",
            "Programming Language :: Python :: 3 :: Only",
            "Topic :: Scientific/Engineering",
            "Topic :: Scientific/Engineering :: Artificial Intelligence",
            "Topic :: Software Development",
            "Topic :: Software Development :: Libraries",
            "Topic :: Software Development :: Libraries :: Python Modules",
        ],
        package_dir={
            "pytorch_kfp_components":
                os.path.join(relative_directory, "pytorch_kfp_components")
        },
        packages=find_packages(where=relative_directory),
    )



#!/usr/bin/env/python3
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Training Component class."""

from typing import Optional, Dict
from pytorch_kfp_components.components.trainer.executor import Executor
from pytorch_kfp_components.components.base.base_component import BaseComponent
from pytorch_kfp_components.types import standard_component_specs


class Trainer(BaseComponent):  #pylint: disable=too-few-public-methods
    """Initializes the Trainer class."""

    def __init__(  # pylint: disable=R0913
        self,
        module_file: Optional = None,
        data_module_file: Optional = None,
        data_module_args: Optional[Dict] = None,
        module_file_args: Optional[Dict] = None,
        trainer_args: Optional[Dict] = None,
    ):
        """Initializes the PyTorch Lightning training process.

        Args:
            module_file : Inherit the model class for training.
            data_module_file : From this the data module class is inherited.
            data_module_args : The arguments of the data module.
            module_file_args : The arguments of the model class.
            trainer_args : arguments specific to the PTL trainer.

        Raises:
            NotImplementedError : If mandatory args;
            module_file or data_module_file is empty.
        """

        super(Trainer, self).__init__()  # pylint: disable=R1725
        input_dict = {
            standard_component_specs.TRAINER_MODULE_FILE: module_file,
            standard_component_specs.TRAINER_DATA_MODULE_FILE: data_module_file,
        }

        output_dict = {}

        exec_properties = {
            standard_component_specs.TRAINER_DATA_MODULE_ARGS: data_module_args,
            standard_component_specs.TRAINER_MODULE_ARGS: module_file_args,
            standard_component_specs.PTL_TRAINER_ARGS: trainer_args,
        }

        spec = standard_component_specs.TrainerSpec()
        self._validate_spec(
            spec=spec,
            input_dict=input_dict,
            output_dict=output_dict,
            exec_properties=exec_properties,
        )

        if module_file and data_module_file:
            # Both module file and data module file are present

            Executor().Do(
                input_dict=input_dict,
                output_dict=output_dict,
                exec_properties=exec_properties,
            )

            self.ptl_trainer = output_dict.get(
                standard_component_specs.PTL_TRAINER_OBJ, "None"
            )
            self.output_dict = output_dict
        else:
            raise NotImplementedError(
                "Module file and Datamodule file are mandatory. "
                "Custom training methods are yet to be implemented"
            )



#!/usr/bin/env/python3
#
# Copyright (c) Facebook, Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Visualization Component Class."""
from pytorch_kfp_components.types import standard_component_specs
from pytorch_kfp_components.components.base.base_component import BaseComponent
from pytorch_kfp_components.components.visualization.executor import Executor


class Visualization(BaseComponent):  # pylint: disable=R0903
    """Visualization Component Class."""

    def __init__(  # pylint: disable=R0913
        self,
        mlpipeline_ui_metadata=None,
        mlpipeline_metrics=None,
        confusion_matrix_dict=None,
        test_accuracy=None,
        markdown=None,
    ):
        """Initializes the Visualization component.

        Args:
            mlpipeline_ui_metadata : path to save ui metadata
            mlpipeline_metrics : metrics to be uploaded
            confusion_metrics_dict : dict for the confusion metrics
            test_accuracy : test accuracy of the model
            markdown : markdown dictionary
        """
        super(BaseComponent, self).__init__()  # pylint: disable=E1003

        input_dict = {
            standard_component_specs.VIZ_CONFUSION_MATRIX_DICT:
                confusion_matrix_dict,
            standard_component_specs.VIZ_TEST_ACCURACY: test_accuracy,
            standard_component_specs.VIZ_MARKDOWN: markdown,
        }

        output_dict = {}

        exec_properties = {
            standard_component_specs.VIZ_MLPIPELINE_UI_METADATA:
                mlpipeline_ui_metadata,
            standard_component_specs.VIZ_MLPIPELINE_METRICS:
                mlpipeline_metrics,
        }

        spec = standard_component_specs.VisualizationSpec()
        self._validate_spec(
            spec=spec,
            input_dict=input_dict,
            output_dict=output_dict,
            exec_properties=exec_properties,
        )
        if markdown:
            self._validate_markdown_spec(spec=spec, markdown_dict=markdown)

        if confusion_matrix_dict:
            self._validate_confusion_matrix_spec(
                spec=spec, confusion_matrix_dict=confusion_matrix_dict
            )

        Executor().Do(
            input_dict=input_dict,
            output_dict=output_dict,
            exec_properties=exec_properties,
        )

        self.output_dict = output_dict

    def _validate_markdown_spec(
        self, spec: standard_component_specs, markdown_dict: dict
    ):
        """Vaildates markdown specs type"""
        for key in spec.MARKDOWN_DICT:
            if key not in markdown_dict:
                raise ValueError(f"Missing mandatory key - {key}")
            if key in markdown_dict:
                self._type_check(
                    actual_value=markdown_dict[key],
                    key=key,
                    spec_dict=spec.MARKDOWN_DICT,
                )

    def _validate_confusion_matrix_spec(
        self, spec: standard_component_specs, confusion_matrix_dict: dict
    ):
        """Validates confusion matrix specs type"""
        for key in spec.CONFUSION_MATRIX_DICT:
            if key not in confusion_matrix_dict:
                raise ValueError(f"Missing mandatory key - {key}")
            if key in confusion_matrix_dict:
                self._type_check(
                    actual_value=confusion_matrix_dict[key],
                    key=key,
                    spec_dict=spec.CONFUSION_MATRIX_DICT,
                )
